# Generated by roxygen2: do not edit by hand

S3method("$","box$mod")
S3method("$","box$ns")
S3method(as.character,"box$mod_info")
S3method(as.character,"box$pkg_info")
S3method(as.character,"box$spec")
S3method(path,"box$mod")
S3method(path,"box$ns")
S3method(print,"box$info")
S3method(print,"box$mod")
S3method(print,"box$spec")
S3method(spec_name,"box$mod_spec")
S3method(spec_name,"box$pkg_spec")
export(export)
export(file)
export(help)
export(name)
export(purge_cache)
export(register_S3_method)
export(reload)
export(script_path)
export(set_script_path)
export(topenv)
export(unload)
export(use)
useDynLib(box, .registration = TRUE)
